"""Helper classes for scripts for cluster support packages."""

from typing import List

from cyberfusion.ClusterSupport._interfaces import (
    APIObjectInterface,
    sort_lists,
)

ENDPOINT_FIREWALL_GROUPS = "/api/v1/firewall-groups"
MODEL_FIREWALL_GROUPS = "firewall_groups"


class FirewallGroup(APIObjectInterface):
    """Represents object."""

    _TABLE_HEADERS = [
        "ID",
        "Name",
        "IP Networks",
        "Cluster",
    ]
    _TABLE_HEADERS_DETAILED: list = []

    _TABLE_FIELDS = [
        "id",
        "name",
        "ip_networks",
        "_cluster_label",
    ]
    _TABLE_FIELDS_DETAILED: list = []

    @sort_lists  # type: ignore[misc]
    def _set_attributes_from_model(
        self,
        obj: dict,
    ) -> None:
        """Set class attributes from API output."""
        self.json_body = obj

        self.id = obj["id"]
        self.name = obj["name"]
        self.ip_networks = obj["ip_networks"]
        self.cluster_id: int = obj["cluster_id"]
        self.created_at = obj["created_at"]
        self.updated_at = obj["updated_at"]

        self.cluster = self.support.get_clusters(id_=self.cluster_id)[0]

        self._cluster_label = self.cluster._label

    def create(
        self,
        *,
        name: str,
        ip_networks: List[str],
        cluster_id: int,
    ) -> None:
        """Create object."""
        url = ENDPOINT_FIREWALL_GROUPS
        data = {
            "name": name,
            "ip_networks": ip_networks,
            "cluster_id": cluster_id,
        }

        self.support.request.POST(url, data)
        response = self.support.request.execute()

        self._set_attributes_from_model(response)

        self.support.firewall_groups.append(self)

    def update(self) -> None:
        """Update object."""
        url = f"{ENDPOINT_FIREWALL_GROUPS}/{self.id}"
        data = {
            "id": self.id,
            "name": self.name,
            "ip_networks": self.ip_networks,
            "cluster_id": self.cluster_id,
        }

        self.support.request.PATCH(url, data)
        response = self.support.request.execute()

        self._set_attributes_from_model(response)

    def delete(self) -> None:
        """Delete object."""
        url = f"{ENDPOINT_FIREWALL_GROUPS}/{self.id}"

        self.support.request.DELETE(url)
        self.support.request.execute()

        self.support.firewall_groups.remove(self)
